"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.escapeXML = exports.renameAsset = exports.renameScreenshot = exports.getSuite = exports.getEnv = exports.getArgs = exports.prepareNpmEnv = exports.getNpmConfig = exports.hasNodeModulesFolder = exports.rebuildNpmDependencies = exports.installNpmDependencies = exports.setUpNpmConfig = exports.getDefaultRegistry = exports.loadRunConfig = exports.shouldRecordVideo = exports.getAbsolutePath = void 0;
// vim: tabstop=2 shiftwidth=2 expandtab
const path_1 = __importDefault(require("path"));
const fs_1 = __importDefault(require("fs"));
const lodash_1 = __importDefault(require("lodash"));
const yargs_1 = __importDefault(require("yargs/yargs"));
const npm_1 = __importDefault(require("./npm"));
const DEFAULT_REGISTRY = 'https://registry.npmjs.org';
function getAbsolutePath(pathToDir) {
    if (path_1.default.isAbsolute(pathToDir)) {
        return pathToDir;
    }
    return path_1.default.join(process.cwd(), pathToDir);
}
exports.getAbsolutePath = getAbsolutePath;
function shouldRecordVideo() {
    const isVideoRecording = process.env.SAUCE_CYPRESS_VIDEO_RECORDING;
    if (isVideoRecording === undefined) {
        return true;
    }
    const videoOption = String(isVideoRecording).toLowerCase();
    return videoOption === 'true' || videoOption === '1';
}
exports.shouldRecordVideo = shouldRecordVideo;
let runConfig;
function loadRunConfig(cfgPath) {
    if (runConfig) {
        return runConfig;
    }
    if (fs_1.default.existsSync(cfgPath)) {
        runConfig = require(cfgPath);
        return runConfig;
    }
    throw new Error(`Runner config (${cfgPath}) unavailable.`);
}
exports.loadRunConfig = loadRunConfig;
function getDefaultRegistry() {
    return process.env.SAUCE_NPM_CACHE || DEFAULT_REGISTRY;
}
exports.getDefaultRegistry = getDefaultRegistry;
async function setUpNpmConfig(nodeCtx, userConfig) {
    console.log('Preparing npm environment');
    const defaultConfig = {
        json: false,
        save: false,
        audit: false,
        fund: false,
        noproxy: 'registry.npmjs.org',
        cafile: process.env.CA_FILE || null,
        'package-lock': false,
        'strict-ssl': true,
        registry: getDefaultRegistry(),
        'update-notifier': false
    };
    await npm_1.default.configure(nodeCtx, Object.assign({}, defaultConfig, userConfig));
}
exports.setUpNpmConfig = setUpNpmConfig;
async function installNpmDependencies(nodeCtx, packageList) {
    const packages = Object.entries(packageList).map(([k, v]) => (`${k}@${v}`));
    console.log(`\nInstalling packages: ${packages.join(' ')}`);
    await npm_1.default.install(nodeCtx, packageList);
}
exports.installNpmDependencies = installNpmDependencies;
async function rebuildNpmDependencies(nodeCtx, path) {
    console.log(`\nRebuilding packages:`);
    if (path) {
        await npm_1.default.rebuild(nodeCtx, '--prefix', path);
    }
    else {
        await npm_1.default.rebuild(nodeCtx);
    }
}
exports.rebuildNpmDependencies = rebuildNpmDependencies;
// Check if node_modules already exists in provided project
function hasNodeModulesFolder(runCfg) {
    const projectFolder = path_1.default.dirname(runCfg.path);
    // Docker: if sauce-runner.json is in home, node_module won't be users'
    //         but the one of the runner => Discard it.
    if (!process.env.SAUCE_VM && projectFolder === process.env.HOME) {
        return false;
    }
    // Assumption: sauce-runner.json is at the root level of project folder.
    // With this location, the presence of node_modules can be checked.
    const nodeModulePath = path_1.default.join(projectFolder, 'node_modules');
    try {
        const st = fs_1.default.statSync(nodeModulePath);
        if (st && st.isDirectory()) {
            return true;
        }
    }
    catch (e) { }
    return false;
}
exports.hasNodeModulesFolder = hasNodeModulesFolder;
function getNpmConfig(runnerConfig) {
    if (runnerConfig.npm === undefined) {
        return {};
    }
    return {
        registry: runnerConfig.npm.registry || getDefaultRegistry(),
        'strict-ssl': runnerConfig.npm.strictSSL !== false,
        // Setting to false to avoid dealing with the generated file.
        'package-lock': runnerConfig.npm.packageLock === true
    };
}
exports.getNpmConfig = getNpmConfig;
async function prepareNpmEnv(runCfg, nodeCtx) {
    const data = { install: { duration: 0 }, setup: { duration: 0 } };
    const npmMetrics = { name: 'npm_metrics.json', data };
    const packageList = runCfg?.npm?.packages || {};
    const nodeModulesPresent = hasNodeModulesFolder(runCfg);
    const npmConfig = getNpmConfig(runCfg);
    let startTime = (new Date()).getTime();
    await setUpNpmConfig(nodeCtx, npmConfig);
    let endTime = (new Date()).getTime();
    npmMetrics.data.setup = { duration: endTime - startTime };
    // rebuild npm packages if node_modules provided
    if (nodeModulesPresent) {
        console.log(`Detected node_modules, running npm rebuilding.`);
        const projectPath = path_1.default.dirname(runCfg.path);
        startTime = (new Date()).getTime();
        await rebuildNpmDependencies(nodeCtx, projectPath);
        endTime = (new Date()).getTime();
        npmMetrics.data.rebuild = { duration: endTime - startTime };
    }
    if (Object.keys(packageList).length === 0) {
        return npmMetrics;
    }
    // Ensure version is a string value as NPM only accepts strings.
    const fixedPackageList = Object.fromEntries(Object.entries(packageList).map(([k, v]) => [k, String(v)]));
    // install npm packages
    startTime = (new Date()).getTime();
    await installNpmDependencies(nodeCtx, fixedPackageList);
    endTime = (new Date()).getTime();
    npmMetrics.data.install = { duration: endTime - startTime };
    return npmMetrics;
}
exports.prepareNpmEnv = prepareNpmEnv;
let args = null;
function getArgs() {
    if (args) {
        return args;
    }
    const argv = (0, yargs_1.default)(process.argv.slice(2))
        .command('$0', 'the default command')
        .option('runCfgPath', {
        alias: 'r',
        type: 'string',
        description: 'Path to sauce runner json',
    })
        .option('suiteName', {
        alias: 's',
        type: 'string',
        description: 'Select the suite to run'
    })
        .demandOption(['runCfgPath', 'suiteName'])
        .argv;
    const { runCfgPath, suiteName } = argv;
    const nodeBin = process.argv[0];
    args = { nodeBin, runCfgPath, suiteName };
    return args;
}
exports.getArgs = getArgs;
function getEnv(suite) {
    let env = {};
    if (lodash_1.default.isObject(suite.env)) {
        env = { ...env, ...suite.env };
    }
    if (lodash_1.default.isObject(suite.config?.env)) {
        env = { ...env, ...suite?.config?.env };
    }
    // If the variable starts with $, pull that environment variable from the process
    for (const [name, value] of lodash_1.default.toPairs(env)) {
        const expectedValue = process.env[value.substring(1)];
        if (value.startsWith('$') && expectedValue) {
            env[name] = expectedValue;
        }
    }
    return env;
}
exports.getEnv = getEnv;
function getSuite(runConfig, suiteName) {
    return runConfig.suites.find((testSuite) => testSuite.name === suiteName);
}
exports.getSuite = getSuite;
// renameScreenshot renames screenshot.
// nested/example.test.js/screenshot.png will be renamed to nested__example.test.js__screenshot.png
// example.test.js/screenshot.png will be renamed to example.test.js__screenshot.png
function renameScreenshot(specFile, oldFilePath, folderName, fileName) {
    const newName = path_1.default.join(folderName, specFile.replace(path_1.default.sep, '__') + '__' + fileName);
    fs_1.default.renameSync(oldFilePath, newName);
    return newName;
}
exports.renameScreenshot = renameScreenshot;
// renameAsset renames asset.
// nested/example.test.js.xml will be renamed to nested__example.test.js.xml
// example.test.js.xml will not be renamed and stay example.test.js.xml
function renameAsset({ specFile, oldFilePath, resultsFolder }) {
    const splittedSpecFile = specFile.split(path_1.default.sep);
    if (splittedSpecFile.length < 2) {
        return oldFilePath;
    }
    // create new file name
    const newFile = splittedSpecFile.slice(0, splittedSpecFile.length).join('__');
    const newFilePath = path_1.default.join(resultsFolder, newFile);
    fs_1.default.renameSync(oldFilePath, newFilePath);
    return newFilePath;
}
exports.renameAsset = renameAsset;
function escapeXML(val) {
    return val.replace(/[<>&'"]/g, (c) => {
        switch (c) {
            case '<': return '&lt;';
            case '>': return '&gt;';
            case '&': return '&amp;';
            case '\'': return '&apos;';
            case '"': return '&quot;';
        }
        return c;
    });
}
exports.escapeXML = escapeXML;
//# sourceMappingURL=utils.js.map